package jcontrol.demos.uielements;

/*
 * Typewalker.java
 *
 * Copyright (c) 2002 by DOMOLOGIC Home-Automation GmbH,
 * Rebenring 33, D-38106 Braunschweig, Germany
 * All rights reserved.
 *
 * This software is for demonstration purposes according to the JControl specifications
 * published by Domologic GmbH. This is public release, you may change the
 * source code only for your own purposes.
 * Please do not delete this comment.
 */
import jcontrol.io.*;

/**
 * Virtual keyboard using one large area scrolling over the screen.
 * 
 * @version 1.0
 * @author  Marcus Timmermann
 */
public class Typewalker {

    private static final int MINLETTER = 32;
    private static final int MAXLETTER = 97;
    private static Display lcd;
    private static Keyboard keys;
    private static char currentChar = (char)97;  // ENTER   
    private static int letter = -1;
           
    /**
     * Displays the virtual keyboard and returns the edited text.
     * @param word text to edit or empty for a new text
     * @param d the Display to display the keyboard
     * @param k the (physical) keyboard to read user commands
     * @return String the edited text
     */
    public static String getText(String word, Display d, Keyboard k) {
        if (word==null) {
            word = "";
        }
        Typewalker.lcd = d;
        Typewalker.keys = k;
        try {
            lcd.setFont(new Resource("Times13.jcfd"));
        } catch (java.io.IOException e) {
        }
        int ypos = 0;  // Selektierte Zeile
        int xpos = 0;  // Verschiebung des Bildes aus der Mitte -> - , <- +
        int oldxpos = -1;
        int oldypos = -1;  
        lcd.setDrawMode(Display.NORMAL);
        lcd.drawRect(0,0,127,16);
        boolean isSticker=false;
        mainloop:for (;;) {
            {
                int l = lcd.getTextWidth(word);                
                if (l>124) {
                    lcd.clearRect(1,1,125, 13);
                    lcd.drawString(word, 2,1,124,13,l-124,0);
                } else {
                    lcd.drawString(word, 2, 1);
                }                
            }
            ypos = ((currentChar-MINLETTER)%3); // Selektierte Zeile
            xpos = ((currentChar-MINLETTER)/3); // Verschiebung des Bildes aus der Mitte -> - , <- +
            if (xpos!=oldxpos) {
                // Hintergrundbild malen
                if (xpos<4) { // wenn links noch Platz ist                                 
                    try {
                        Resource image = new Resource("typewalker.jcif");
                        int xoff = (4-xpos)*14; 
                        lcd.drawImage(image, xoff, 17, 127-xoff, 46, 0,0);
                        lcd.drawImage(image, 0, 17, xoff, 46, 308-xoff,0);
                    } catch (java.io.IOException e) {
                    } 
                } else if (xpos>=4 && xpos<=17) {
                    try {
                        Resource image = new Resource("typewalker.jcif");
                        int xoff = (xpos-4)*14;
                        lcd.drawImage(image, 0, 17, 127, 46, xoff,0);                    
                    } catch (java.io.IOException e) {
                    }
                } else { // wenn rechts noch Platz ist
                    try {
                        Resource image = new Resource("typewalker.jcif");
                        int xoff = (xpos-4)*14;
                        lcd.drawImage(image, 0, 17, 308-xoff, 46, xoff,0);                    
                        lcd.drawImage(image, 308-xoff, 17, 127-(308-xoff), 46, 0,0);
                    } catch (java.io.IOException e) {
                    }
                }           
                lcd.drawRect(55,16,17,48);
                oldxpos = xpos;
                oldypos = -1; // damit das Auswahlrechteck wieder hingemalt wird
            }
            if (ypos!=oldypos) {
                lcd.setDrawMode(Display.XOR);
                if (oldypos>-1) {
                    lcd.fillRect(57,19+(oldypos*14),13,14); 
                }
                lcd.fillRect(57,19+(ypos*14),13,14);
                oldypos = ypos;
                lcd.setDrawMode(Display.NORMAL);                
            }
            char key = keys.read();
            switch (key) {
                case 'L':
                    isSticker=false;
                    currentChar-=3;
                    break;
                case 'U':
                    currentChar-=(isSticker)?3:1;
                    break;
                case 'u':
                    isSticker=true;
                    currentChar--;
                    break;
                case 'R':
                    isSticker=false;
                    currentChar+=3;
                    break;
                case 'D':
                    currentChar+=(isSticker)?3:1;
                    break;
                case 'd':                
                    isSticker=true;
                    currentChar++;
                    break;
                case 'S':
                    if (currentChar==MAXLETTER) {
                        return word;
                    } else if (currentChar==MAXLETTER-1) {
                        if (word.length()>1) {
                            word = word.substring(0, word.length()-1);                            
                        } else {
                            word = "";
                        }
                        lcd.clearRect(1,1,125, 13);
                    } else {
                        word = word.concat(new String (new byte[]{(byte)currentChar}));
                    }


            }
            if (currentChar<MINLETTER) currentChar = MAXLETTER;
            if (currentChar>MAXLETTER) currentChar = MINLETTER;
        } // mainloop


    }

}
